
#ifndef __GENERATOR_HPP__
#define __GENERATOR_HPP__

#include "resources.hpp"


// The Generator class provides the target position as a function of time
// that we want to attain in the operational space.
// The polymorphism allows to easily implement different trajectories.
class Generator : public yarp::os::RateThread
{
protected:
    double Ax;
    double Ay;
    double t0;

    Vector *pd;

    Resource &res_gen;

    // this is a pure abstract method (you notice it from the '=0')
    // which is required to be defined within the children classes
    virtual void generate(const double t)=0;

public:
    Generator(Resource &_res_gen, const double _Ax, const double _Ay) :
             RateThread(SAMPLE_TIME), res_gen(_res_gen), Ax(_Ax), Ay(_Ay)
    {
        // init the desired target at the starting point
        pd=new Vector(2);
        (*pd)[0]=Ax;
        (*pd)[1]=Ay;

        res_gen.set(*pd);
    }

    virtual bool threadInit()
    {        
        t0=yarp::os::Time::now();
        return true;
    }

    virtual void run()
    {
        // the correct generate() method will be called
        generate(yarp::os::Time::now()-t0);
    }    

    virtual void threadRelease()
    {        
        delete pd;
    }
};


// It yields just a fixed point
class QuietGenerator : public Generator
{
protected:
    virtual void generate(const double t)
    {
        (*pd)[0]=Ax;
        (*pd)[1]=Ay;

        res_gen.set(*pd);
    }

public:
    QuietGenerator(Resource &_res_gen, const double _Ax, const double _Ay) :
                   Generator(_res_gen,_Ax,_Ay) { }
};


// It yields a point moving on a horizontal line, back and forth
class HorizontalGenerator : public Generator
{
protected:
    virtual void generate(const double t)
    {
        (*pd)[0]=Ax*cos(2*PI*GEN_FREQ*t);
        (*pd)[1]=Ay;

        res_gen.set(*pd);
    }

public:
    HorizontalGenerator(Resource &_res_gen, const double _Ax, const double _Ay) :
                       Generator(_res_gen,_Ax,_Ay) { }
};


// It yields a point moving on the diagonal line y=x, back and forth
class DiagonalGenerator : public Generator
{
protected:
    virtual void generate(const double t)
    {
        (*pd)[0]=Ax*cos(2*PI*GEN_FREQ*t);
        (*pd)[1]=(*pd)[0];

        res_gen.set(*pd);
    }

public:
    DiagonalGenerator(Resource &_res_gen, const double _A) :
                     Generator(_res_gen,_A,_A) { }
};


// It yields a point moving on an ellipse (or circle)
class EllipseGenerator : public Generator
{
protected:
    virtual void generate(const double t)
    {
        (*pd)[0]=Ax*cos(2*PI*GEN_FREQ*t);
        (*pd)[1]=Ay*sin(2*PI*GEN_FREQ*t);

        res_gen.set(*pd);
    }

public:
    EllipseGenerator(Resource &_res_gen, const double _Ax, const double _Ay) :
                    Generator(_res_gen,_Ax,_Ay) { }
};


#endif


