
#ifndef __MOTOR_HPP__
#define __MOTOR_HPP__

#include <yarp/os/RateThread.h>
#include <yarp/os/Semaphore.h>

#include "resources.hpp"


// The class Motor simulates the motor behaviour in terms of
// the transfer function F(z^-1)=q(z^-1)/qp(z^-1), where q(z^-1)
// is the motor joints vector and qp(z^-1) is the corresponding
// velocities vector.
// Remind that q=[theta; r].
// 
// Here F(z^-1) is simply a discrete integrator
// implemented with the Tustin formula:
// F(z^-1)=Ts/2 * (1+z^-1) / (1-z^-1)
class Motor : public yarp::os::RateThread
{
protected:
    Vector *qpOld;

    const Resource &res_ctrl;
    Resource &res_mot;

    double t0;
    
public:
    Motor(Resource &_res_ctrl, Resource &_res_mot, const Vector &q0) :
         RateThread(SAMPLE_TIME), res_ctrl(_res_ctrl), res_mot(_res_mot)
    {
        res_mot.set(q0);        // init starting joints position

        qpOld=new Vector(2);    // starting velocities = 0.0
        (*qpOld)[0]=0.0;
        (*qpOld)[1]=0.0;
    }

    virtual bool threadInit()
    {
        t0=yarp::os::Time::now();
        return true;
    }

    virtual void run()
    {
        const double t=yarp::os::Time::now();
        const double dt=t-t0;

        Vector q=res_mot.get();
        Vector qp=res_ctrl.get();

        // the Tustin formula
        Vector qNew=q+(qp+*qpOld)*(dt/2.0);

        // wrap around the rotation;
        // it is assumed that velocity is not
        // so high to impose in one step a delta
        // angle greater than 2*PI
        if (qNew[0]>2*PI)
            qNew[0]-=2*PI;
        else if (qNew[0]<0)
            qNew[0]+=2*PI;

        res_mot.set(qNew);

        *qpOld=qp;
        t0=t;
    }    

    virtual void threadRelease()
    {
        delete qpOld;
    }
};


#endif

